package FAIR::Data;
$FAIR::Data::VERSION = '0.230';

# ABSTRACT:  a set of packages that assist with creating infrastructures that conform to the FAIR Data Principles

__END__

=pod

=encoding UTF-8

=head1 NAME

FAIR::Data - a set of packages that assist with creating infrastructures that conform to the FAIR Data Principles

=head1 VERSION

version 0.230

=head1 Title - FAIR::Data

a set of packages that assist with creating infrastructures that conform to the FAIR Data Principles

Findable

Acessible

Interoperable

Reusable

=head 1 Use -  do not use this package (FAIR::Data)

This package is purely informative.  When writing your code you should use one of the more specific packages within this distribution, depending on your specific task.

e.g. use FAIR::Accessor to assist with creating a FAIR Accessor Server, compliant with the Linked Data Platform specification.

=head1 AUTHOR

Mark Denis Wilkinson (markw [at] illuminae [dot] com)

=head1 COPYRIGHT AND LICENSE

This software is Copyright (c) 2015 by Mark Denis Wilkinson.

This is free software, licensed under:

  The Apache License, Version 2.0, January 2004

=cut
